
//genesis
// cable.g - main script

// Specific resistances and capacitance - old values
// Kohm-cm^2
//float RM = 10.0
// uF/cm^2
//float CM = 1.0
// Kohm-cm
//float RA = 0.05

float RM = 3.33333333
float CM = 1.0
float RA = 0.03
float PI = 3.14159
float RM_EX = {RM}

//==================================================
// global variables used in the simulation
//==================================================

// start with one passive cable compartment
int NCableSects = 0
// number of segments to use for axon
int NAxonSegs = 21	
// use the exponential numerical integration method 
int IntegMethod = 0
// default time to run simulation (msecs)
float tmax = 50
// default integration time step  (msecs)
float dt = 0.005
// resting membrane potential
float EREST = -70.0


// Voltages and stuff
float EREST_ACT = EREST
float ELEAK = EREST_ACT + 10.6
float ENA = 45.0
float EK = -82.0

// compartment dimensions (cm.)
float soma_l = 50e-4
float dend_l = .1
float dend_d = 2e-4
float axon_l = .05 	
float axon_d = 2e-5

// default (initial) parameters for current injection
// msec
float injwidth = 0.5
float injdelay = 0
// uA (100 nA)
float injcurrent = -1e-3

// default (initial) parameters for axonal current injection
// msec
float axinjwidth = 0.5
float axinjdelay = 0
// uA (100 nA)
float axinjcurrent = -1e-3

// default (initial) parameters synaptic input
// msec
float syndelay1 = 2
float syndelay2 = 2
// choose width and interval to give a single spike
float synwidth = 1000
float syninterval = 100000

// milliSiemen
float gsyn1 = 3e-6
float gsyn2 = 3e-6
// milliseconds
float tsyn1 = 1
float tsyn2 = 1
// mV above resting potential (defined as zero)
float Vsyn1 = 60
float Vsyn2 = -120
// for comparison
// Vsyn = 125 ==> Na channel (+55 mV with Erest = -70)
// Vsyn = -12 ==>  K channel (-82 mV with Erest = -70)


//==================================================
//                  STARTUP SCRIPTS
//==================================================
include compartment.g 
include inputs.g 
//include logifier.g // constructs lookup tables for log plots
include setcolors.g // set colors of things

include axon.g	    // create active soma and axon

include view.g // graphics for propagation along the cable

include addcable.g // functions to add and delete cable sections

include forms.g // Control Panel and Graphs

include hhchan1.g	// HH channel descriptions

//=========================================================================
//  Some functions used in the main script

//  SendPlotMsgs  - sends messages to the graphics forms

//  ResetGlobals  - resets the values of the global variables to defaults

//  ResetDefaults - similar to the main script, but graph forms pre-exist
//=========================================================================

function SendPlotMsgs
  pushe /output/somagraphs
  addmsg /cell/soma somaVm_grf PLOT Vm *soma *black
  addmsg /cell/syn_channel /output/condgraphs/channel_Gk PLOT Gk *Gk *blue
  addmsg /cell/axon[{0}]   /output/axon_plot/voltage PLOT Vm *"axon0 "  *black  
  addmsg /cell/axon[{5}]   /output/axon_plot/voltage PLOT Vm *"axon5 "  *blue  
  addmsg /cell/axon[{10}]   /output/axon_plot/voltage PLOT Vm *axon10  *red  
  addmsg /cell/axon[{15}]  /output/axon_plot/voltage PLOT Vm *axon15 *green4  
  addmsg /cell/axon[{20}]  /output/axon_plot/voltage PLOT Vm *axon20 *orange3  
  pope
end// SendPlotMsgs


function ResetGlobals
    // start with no passive cable compartments
    NCableSects = 0
    IntegMethod = 0
    // default time to run simulation (msecs)
    tmax = 50
    // default integration time step  (msecs)
    dt = 0.005
    // Kohm-cm^2
    //RM = 10.0
    RM = 3.333333333
    // uF/cm^2
    CM = 1.0
    // Kohm-cm
    //RA = 0.05
    RA = 0.03
    soma_l = 50e-4
    dend_l = .1
    dend_d = 2e-4
    axon_l = .05  	
    axon_d = 2e-5
end

function ResetDefaults
    stop
    // in case we are plotting cable sections
    remove_cable_plots
    delete /cell
    delete /input
    delete /output/control
    delete /output/param_menu
    delete /output/inject_menu
    delete /output/synapse_menu
    delete /output/method_menu
    delete /output/plot_menu
    // re-initialize global constants
    ResetGlobals
    // simulation time step in msec
    setclock 0 {dt}
    create neutral /cell
    make_excite_compartment /cell/soma {soma_l} {soma_l} {ELEAK}
    // inputs.g
    create_axon				//axon.g
    makeinputs
    loadcontrol
    add_cable    
    SendPlotMsgs
    adjustspike
    check
    colorize
    reset
end

//=========================================================================

//                         MAIN SCRIPT - Cable

//=========================================================================

//==================================================
// check and initialize the simulation
//==================================================

// set the prompt
setprompt "Cable !"
// simulation time step in msec
setclock 0 {dt}
					// max size for reasonable accuracy
// output interval
setclock 1 0.05

create neutral /cell
make_excite_compartment /cell/soma {soma_l} {soma_l} {ELEAK}
create_axon						//axon.g
// inputs.g
makeinputs
// in forms.g (This is done after the elements are created)
loadgraphs
loadcontrol
loadhelp
add_cable
SendPlotMsgs
adjustspike

check
colorize
reset

echo ==================
echo Simulation loaded.
echo ==================
