
//genesis
/*   inputs.g for cable tutorial
---------------------------------------------------------------------------
makeinputs
     used to provide pulsed current injection to a compartment and
     pulsed spike trains to the synapse

set_inj_timing(path,delay,width)
     used to set the injection timing parameters

add_injection(comp_no)
     checks for valid compartment no., deletes message to previous
     compartment, and sends message to inject {comp_no}

set_injection
     gets info from form defined in forms.g and uses above functions to
     set injection parameters
     
setspikes(chan,delay,width,interval,mof,mode)
     used in makeinputs and forms.g to set the spike train parameters
     for a pulsegen object

set_chan_params(channel,gysn,tsyn,Vsyn)
	set parameters for a synaptic channel
     
add_syn_input(comp_no)

set_inputs
     used for setting synaptically activated channel input parameters

---------------------------------------------------------------------------

Modiied by Ed Vigmond 4/9/95
	an axonal current injection was added
*/

function set_inj_timing(path,delay, width)
    str path    
    float delay, width
    setfield /input/{path} width1 {width} delay1 {delay}  \
        baselevel 0.0 trig_mode 0 delay2 100000
    // free run mode with very long delay for 2nd pulse (non-repetitive)
    // level1 is set by the inj_toggle function
end

function inj_toggle(path,state)// toggles current injection ON/OFF
    str path
    int state

    if ( {state} == 1)
        // ON
        setfield /input/{path} level1 1.0
    else
        // OFF
        setfield /input/{path} level1 0.0
    end
end

function add_injection(comp_no)
   // compartment # for injection - #0 = Soma
   // Modified to allow for space clamp when all is specified as copmpartment
   str comp_no
   int n
   
   // Add space clamp -- do it simply, remove all messages then add one
   // from the current injection to each compartment
   if( ({comp_no} == "all") || ({comp_no} == "ALL") )
   		echo "Space clamp initiated"
		// delete old messages
		while(({getmsg /input/injectpulse/injcurr -outgoing -count}) > 0)
		  n = {getmsg /input/injectpulse/injcurr -outgoing -count}
		  deletemsg /input/injectpulse/injcurr {n-1} -outgoing 
		end
       	addmsg /input/injectpulse/injcurr /cell/soma INJECT output
   		for( n=0; n<NCableSects; n=n+1 )
			addmsg /input/injectpulse/injcurr /cell/cable[{n}] \
							 INJECT output
   		end
   		return
   end

   if ((comp_no) > ({NCableSects}))
      echo "There are only "{NCableSects}" cable sections!"
      return
   end
   
   // delete all old messages 
   while(({getmsg /input/injectpulse/injcurr -outgoing -count}) > 0)
      n = {getmsg /input/injectpulse/injcurr -outgoing -count}
      deletemsg /input/injectpulse/injcurr {n-1} -outgoing 
   end
   
   if ((comp_no) <= 0)   // then inject soma
      addmsg /input/injectpulse/injcurr /cell/soma INJECT output
      echo "Current injection is to the soma"
   else
      addmsg /input/injectpulse/injcurr /cell/cable[{comp_no - 1}]  \
          INJECT output
      echo "Current injection is to cable section number "{comp_no}
   end

end


function add_axon_injection(comp_no)
   // compartment = # for injection 
   str comp_no
   if ((comp_no) > ({NAxonSegs}-1))
      echo "There are only "{NAxonSegs-1}" axon sections!"
      return
   end
   if (({getmsg /input/axinjectpulse/injcurr -outgoing -count}) > 0)
      deletemsg /input/axinjectpulse/injcurr  0 -outgoing 
						// only outgoing message
   end
   if ((comp_no) <= 0)
	comp_no = {0}
   end
   addmsg /input/axinjectpulse/injcurr /cell/axon[{comp_no}]  \
          INJECT output
    echo "Current injection is to axon section number "{comp_no}
end


function set_injection// gets info from form defined in forms.g

   // set axon/dendritic injection
   str dialog = "/output/inject_menu"
   set_inj_timing injectpulse {getfield {dialog}/delay value}  \
       {getfield {dialog}/width value}
   setfield /input/injectpulse/injcurr  \
       gain {getfield {dialog}/inject value}
   add_injection {getfield {dialog}/comp_no value}
   echo  \
       "Injection current = "{getfield {dialog}/inject value}" microamperes"
   echo  \
       "Injection pulse delay = "{getfield {dialog}/delay value}" millisec"
   echo  \
       "Injection pulse width = "{getfield {dialog}/width value}" millisec"
   //set axonal injection
   set_inj_timing axinjectpulse {getfield {dialog}/axdelay value}  \
       {getfield {dialog}/axwidth value}
   setfield /input/axinjectpulse/injcurr  \
       gain {getfield {dialog}/axinject value}
   add_axon_injection {getfield {dialog}/axcomp_no value}
   echo  \
       "Injection current = "{getfield {dialog}/axinject value}" microamperes"
   echo  \
       "Injection pulse delay = "{getfield {dialog}/axdelay value}" millisec"
   echo  \
       "Injection pulse width = "{getfield {dialog}/axwidth value}" millisec"
end


function setspikes(chan, delay, width, interval, trmode)
	str chan
	float period = (delay) - 1
    str pulser
    str spiker
    float delay, width, interval, width2, trmode
    
    pulser = "/input/synpulse"@{chan}
    spiker = {pulser}@"/spiketrain"
    setfield {pulser} width1 1 delay1 0 baselevel 0.0 trig_time 0  \
        delay2 1 level2 0.25 width2 {period} trig_mode {trmode}
    // ed thing
    setfield {spiker} abs_refract 1 output_amp 1 thresh 0.5
end


function set_chan_params(chan, gsyn, tsyn, Vsyn)
   str chan, gsyn, tsyn, Vsyn
   setfield /cell/{chan} Ek {Vsyn}
   setfield /cell/{chan} tau1 {tsyn} tau2 {tsyn}
   setfield /cell/{chan} gmax {gsyn}
end

// links channel which computes channel current (channelC2) with compartment
function link_channel2(channel, compartment)
    addmsg {channel} {compartment} CHANNEL Gk Ek
    addmsg {compartment} {channel} VOLTAGE Vm
end

//modify spike output based on integration step to give constant impulse area
function adjustspike
    setfield /##[TYPE=spike] output_amp {1.0/{getclock 0}}
end

function add_syn_input(comp_no1, comp_no2)
   // compartment # for input - #0 = Soma
   str comp_no
   if (((comp_no1) > ({NCableSects})) || ((comp_no1) > ({NCableSects})))
      echo "There are only "{NCableSects}" cable sections!"
      return
   end
   // NOTE: We could disconnect the channel from the old compartment by
   // deleting all its messages.  If the conductance (Gk) is plotted this
   // won't work, unless we are careful to keep the message numbering staight.
   // To be safe, we locate the right "OUT" message.
    int i, n, nmsg
    nmsg = {getmsg /cell/syn_channel -outgoing -count}
    for (i = 0; i < nmsg; i = i + 1)
        if ( \
            ({getmsg /cell/syn_channel -outgoing -type {i}}) == "CHANNEL" \
            )
            n = i
            break
        else
            n = -1
        end
    end
    if (n >= 0)
	deletemsg /cell/syn_channel  {n} -outgoing
    end
    if (({getmsg /cell/syn_channel -incoming -count}) > 0)
       deletemsg /cell/syn_channel  0 -incoming // input from axon doesn't use msgs
    end
    if (({getfield /output/synapse_menu/syntoggle state}) == 1)
	if ((comp_no1) == 0)	// then input to soma

	  link_channel2 /cell/syn_channel /cell/soma
	  echo "Synaptic input 1 is to the soma"
	else
	  link_channel2 /cell/syn_channel /cell/cable[{comp_no1 - 1}]
	  echo "Synaptic input 1 is to cable section number "{comp_no1}
	end
    end
    nmsg = {getmsg /cell/syn_channel2 -outgoing -count}
    for (i = 0; i < nmsg; i = i + 1)
        if ( \
            ({getmsg /cell/syn_channel2 -outgoing -type {i}}) == "CHANNEL" \
            )
            n = i
            break
        else
            n = -1
        end
    end
    if (n >= 0)
	deletemsg /cell/syn_channel2  {n} -outgoing
    end
    if (({getmsg /cell/syn_channel2 -incoming -count}) > 0)
       deletemsg /cell/syn_channel2  0 -incoming       // input from axon doesn't use msgs

    end
    if (({getfield /output/synapse_menu/syntoggle2 state}) == 1)
	if ((comp_no2) == 0)	// then input to soma

	  link_channel2 /cell/syn_channel2 /cell/soma
	  echo "Synaptic input 2 is to the soma"
	else
	  link_channel2 /cell/syn_channel2 /cell/cable[{comp_no2 - 1}]
	  echo "Synaptic input 2 is to cable section number "{comp_no2}
	end
    end
end

function set_inputs// set parameters for synaptic input from dialog boxes

   str dialog = "/output/synapse_menu"
   setspikes "1" {getfield {dialog}/DELAY1 value} {synwidth} {syninterval}  \
       {getfield {dialog}/traintog state}
   setspikes "2" {getfield {dialog}/DELAY2 value} {synwidth} {syninterval}  \
       {getfield {dialog}/traintog state}
   // set spike amplitudes to 1.0/dt
   adjustspike
   add_syn_input {getfield {dialog}/comp_no value}  \
       {getfield {dialog}/comp_no2 value}
   set_chan_params syn_channel {getfield {dialog}/GSYN1 value}  \
       {getfield {dialog}/TSYN1 value} {getfield {dialog}/ESYN1 value}
   set_chan_params syn_channel2 {getfield {dialog}/GSYN2 value}  \
       {getfield {dialog}/TSYN2 value} {getfield {dialog}/ESYN2 value}
   // after so many changes, we need to reset everything
   reset
end

//=======================================================================
// makeinputs - the main function to set up injection and synaptic input
//=======================================================================

function makeinputs

create neutral /input
//==============================================================
// Make a periodic injection current step for soma and dendrites
//==============================================================
create pulsegen /input/injectpulse
set_inj_timing injectpulse {injdelay} {injwidth}
// set the amplitude (current) with the gain of a differential amplifier
create diffamp /input/injectpulse/injcurr
setfield /input/injectpulse/injcurr saturation 1 gain {injcurrent}
addmsg /input/injectpulse /input/injectpulse/injcurr PLUS output

//==============================================================
// Make a periodic injection current step for axon
//=============================================================
create pulsegen /input/axinjectpulse
set_inj_timing axinjectpulse {axinjdelay} {axinjwidth}
// set the amplitude (current) with the gain of a differential amplifier
create diffamp /input/axinjectpulse/injcurr
setfield /input/axinjectpulse/injcurr saturation 1 gain {axinjcurrent}
addmsg /input/axinjectpulse /input/axinjectpulse/injcurr PLUS output

//==================================================
// spike trains for synaptic input - could be simpler if only a single spike
//==================================================
// pulse to gate spike train
create pulsegen /input/synpulse1
create pulsegen /input/synpulse2
// Use pulse as input to a spike object
create spike /input/synpulse1/spiketrain
create spike /input/synpulse2/spiketrain
setspikes "1" {syndelay1} {synwidth} {syninterval} 1
setspikes "2" {syndelay2} {synwidth} {syninterval} 1
addmsg /input/synpulse1 /input/synpulse1/spiketrain INPUT output
addmsg /input/synpulse2 /input/synpulse2/spiketrain INPUT output
// Now feed these spike trains to an "axon" to allow synaptic weights
create axon /input/synpulse1/axon
create axon /input/synpulse2/axon
addmsg /input/synpulse1/spiketrain /input/synpulse1/axon BUFFER name
addmsg /input/synpulse2/spiketrain /input/synpulse2/axon BUFFER name
// ON
setfield /input/synpulse1 level1 1.0
setfield /input/synpulse2 level1 1.0

// Lastly, make connection to chemically activated dendrite channel
create channelC2 /cell/syn_channel
create channelC2 /cell/syn_channel2
set_chan_params "syn_channel"  {gsyn1} {tsyn1} {Vsyn1}
set_chan_params "syn_channel2" {gsyn2} {tsyn2} {Vsyn2}
connect /input/synpulse1/axon /cell/syn_channel  with synapse
connect /input/synpulse2/axon /cell/syn_channel2 with synapse
// no delay, initial weights 1
setconn /input/synpulse1/axon:0 delay 0 weight 1
// no delay, initial weights 1
setconn /input/synpulse2/axon:0 delay 0 weight 1

// initially, make both injection and synaptic input to the soma
addmsg /input/injectpulse/injcurr /cell/soma INJECT output
link_channel2 /cell/syn_channel  /cell/soma 
link_channel2 /cell/syn_channel2 /cell/soma 
end// makeinputs



// Added by Ed Vigmond
function traintoggle// toggle single pulse/ pulse synaptic input
    float state = {getfield /output/synapse_menu/traintog state}
    setfield /input/synpulse1/ trig_mode {state}
    setfield /input/synpulse2/ trig_mode {state}
    echo Setting /pulsegen trig_mode {state}
end
